from abc import ABC

from cryptomodel.operations import OPERATIONS
from mongoengine import Q
from cryptomodel.cryptostore import user_notification, user_channel, user_transaction
from cryptomodel.cryptostore import user_settings
from cryptodataaccess.helpers import server_time_out_wrapper, if_none_raise, if_none_raise_with_id
from cryptodataaccess.Users.UsersStore import UsersStore
from cryptodataaccess.helpers import do_connect

DATE_FORMAT = "%Y-%m-%d"


class UsersMongoStore(UsersStore, ABC):

    def __init__(self, config, log_error):
        self.configuration = config
        self.log_error = log_error

    def fetch_user_channels(self, user_id):
        return server_time_out_wrapper(self, self.do_fetch_user_channels, user_id)

    def fetch_user_settings(self, user_id):
        return server_time_out_wrapper(self, self.do_fetch_user_settings, user_id)

    def fetch_notifications(self, items_count):
        return server_time_out_wrapper(self, self.do_fetch_notifications, items_count)

    def fetch_notification_by_id(self, id):
        return server_time_out_wrapper(self, self.do_fetch_notification_by_id, id)

    def fetch_user_channel_by_id(self, id):
        return server_time_out_wrapper(self, self.do_fetch_user_channel_by_id, id)

    def insert_notification(self, notification):
        return server_time_out_wrapper(self, self.do_insert_notification, notification)

    def update_notification(self, notification):
        return server_time_out_wrapper(self, self.do_update_notification, notification)

    def fetch_user_settings_by_id(self, id):
        return server_time_out_wrapper(self, self.do_fetch_user_settings_by_id, id)

    def update_user_settings(self, user_settings):
        return server_time_out_wrapper(self, self.do_update_user_settings, user_settings)

    def insert_user_settings(self, user_settings):
        return server_time_out_wrapper(self, self.do_insert_user_settings, user_settings)

    def fetch_user_user_channel_by_id(self, id):
        return server_time_out_wrapper(self, self.do_fetch_user_channel_by_id, id)

    def insert_user_channel(self, user_channel):
        return server_time_out_wrapper(self, self.do_insert_user_channel, user_channel)

    def delete_notification(self, notification, throw_if_does_not_exist=True):
        server_time_out_wrapper(self, self.do_delete_notification, notification, throw_if_does_not_exist)

    def delete_user_settings(self, us):
        server_time_out_wrapper(self, self.do_delete_user_settings, us)

    def do_insert_notification(self, notification):
        do_connect(self.configuration)
        un = user_notification()
        un.user_id = notification.user_id
        un.user_name = notification.user_name
        un.user_email = notification.user_email
        un.notification_type = notification.notification_type
        un.check_every =notification.check_every
        un.start_date = notification.start_date
        un.end_date = notification.end_date
        un.is_active = notification.is_active
        un.channel_type = notification.channel_type
        un.threshold_value = notification.threshold_value
        un.source_id = notification.source_id
        un.operation = notification.operation
        un.save()
        return user_notification.objects(id=un.id).first()

    def do_insert_user_channel(self, in_uc):
        do_connect(self.configuration)
        uc = user_channel()
        uc.user_id = in_uc.user_id
        uc.channel_type = in_uc.channel_type
        uc.chat_id = in_uc.chat_id
        uc.operation = in_uc.operation
        uc.source_id = in_uc.source_id
        uc.save()
        return user_channel.objects(id=uc.id).first()

    def do_insert_user_settings(self, in_uc):
        do_connect(self.configuration)
        us = user_settings()
        us.user_id = in_uc.user_id
        us.preferred_currency = in_uc.preferred_currency
        us.operation = in_uc.operation
        us.source_id = in_uc.source_id
        us.save()
        return user_settings.objects(id=us.id).first()

    def do_update_user_settings(self, user_setting):
        do_connect(self.configuration)
        us = user_setting.objects(id=user_setting.id).first()
        if_none_raise_with_id(user_setting.id, us)
        us.user_id = user_setting.user_id
        us.preferred_currency = user_setting.preferred_currency
        us.save()
        return user_setting.objects(id=id).first()

    def do_update_notification(self, notification):
        do_connect(self.configuration)
        un = user_notification.objects(id=notification.id).first()
        if_none_raise_with_id(notification.id, un)
        un.user_id = notification.user_id
        un.user_name = notification.user_name
        un.user_email = notification.user_email
        un.notification_type = notification.notification_type

        un.check_every = notification.check_every
        un.start_date = notification.start_date
        un.end_date = notification.end_date

        un.is_active = notification.is_active
        un.channel_type = notification.channel_type
        un.threshold_value = notification.threshold_value
        un.source_id = notification.source_id
        un.operation = notification.operation
        un.save()
        return user_notification.objects(id=un.id).first()

    def delete_user_notification_by_source_id(self, source_id, throw_if_does_not_exist=True):
        server_time_out_wrapper(self, self.do_delete_user_notification_by_source_id, source_id,
                                throw_if_does_not_exist)

    def do_delete_user_notification_by_source_id(self, source_id, throw_if_does_not_exist=True):
        do_connect(self.configuration)
        un = user_notification.objects(source_id=source_id).first()
        if throw_if_does_not_exist:
            if_none_raise_with_id(id, un)
        if un is not None:
            un.delete()

    def do_delete_notification(self, notif, throw_if_does_not_exist=True):
        do_connect(self.configuration)
        un = user_notification.objects(id=notif.id).first()
        if throw_if_does_not_exist:
            if_none_raise_with_id(id, un)
        if un is not None:
            un.delete()

    def do_delete_user_settings(self, us):
        do_connect(self.configuration)
        us = user_settings.objects(id=us.id).first()
        if_none_raise_with_id(us.id, us)
        us.delete()

    def do_fetch_user_channels(self, user_id):
        do_connect(self.configuration)
        return user_notification.objects(Q(user_id=user_id))

    def do_fetch_user_settings(self, user_id):
        do_connect(self.configuration)
        return user_settings.objects(Q(user_id=user_id))

    def do_fetch_notifications(self, items_count):
        do_connect(self.configuration)
        return user_notification.objects()[:items_count]

    def do_fetch_user_channel_by_id(self, id):
        do_connect(self.configuration)
        return user_channel.objects(Q(id=id))

    def do_fetch_user_settings_by_id(self, id):
        do_connect(self.configuration)
        return user_settings.objects(Q(id=id))

    def do_fetch_notification_by_id(self, id):
        do_connect(self.configuration)
        return user_notification.objects(Q(id=id))



import json
import pymongo
from cryptodataaccess.helpers import get_url
from cryptodataaccess.config import configure_app
import pytest
import mock

BASE_PATH = 'cryptodataaccess/tests/'


def get_exchange_rates_record():
    with open(BASE_PATH + 'sample_records/exchange_rate.json', 'r') as my_file:
        return json.load(my_file)


def get_prices_record():
    with open(BASE_PATH + 'sample_records/price.json', 'r') as my_file:
        return json.load(my_file)


def get_prices_2020706_record():
    with open(BASE_PATH + 'sample_records/prices_2020706.json', 'r') as my_file:
        return json.load(my_file)


def get_prices_2020731_record():
    with open(BASE_PATH + 'sample_records/prices_2020731.json', 'r') as my_file:
        return json.load(my_file)


def get_prices20200812039_record():
    with open(BASE_PATH + 'sample_records/prices_2038.json', 'r') as my_file:
        return json.load(my_file)


def get_prices20200801T2139_record():
    with open(BASE_PATH + 'sample_records/prices_2039.json', 'r') as my_file:
        return json.load(my_file)


# refactor file use this only
def insert_prices_record_with_method(method_to_get):
    config = configure_app()
    client = pymongo.MongoClient(get_url(config))
    db = client[config.DATABASE]
    prices_col = db["prices"]
    prices_col.insert(method_to_get())


def insert_prices_record():
    config = configure_app()
    client = pymongo.MongoClient(get_url(config))
    db = client[config.DATABASE]
    prices_col = db["prices"]
    prices_col.insert(get_prices_record())


def delete_prices():
    config = configure_app()
    client = pymongo.MongoClient(get_url(config))
    db = client[config.DATABASE]
    prices_col = db["prices"]
    prices_col.delete_many({})


def insert_prices_2020706_record():
    config = configure_app()
    client = pymongo.MongoClient(get_url(config))
    db = client[config.DATABASE]
    prices_col = db["prices"]
    prices_col.insert(get_prices_2020706_record())


def insert_prices_2020731_record():
    config = configure_app()
    client = pymongo.MongoClient(get_url(config))
    db = client[config.DATABASE]
    prices_col = db["prices"]
    prices_col.insert(get_prices_2020731_record())


def insert_exchange_record():
    config = configure_app()
    client = pymongo.MongoClient(get_url(config))
    db = client[config.DATABASE]
    exchange_rates_col = db["exchange_rates"]
    exchange_rates_col.insert(get_exchange_rates_record())


@pytest.fixture(scope='module')
def mock_log():
    with mock.patch("Logger.logger.log_error"
                    ) as _mock:
        _mock.return_value = True
        yield _mock



from cryptomodel.cryptostore import user_notification, user_channel, user_settings
from cryptomodel.operations import OPERATIONS

from cryptodataaccess.Memory import Memory, USER_NOTIFICATIONS_MEMORY_KEY, USER_SETTINGS_MEMORY_KEY, \
    USER_CHANNELS_MEMORY_KEY
from cryptodataaccess.Repository import Repository

DATE_FORMAT = "%Y-%m-%d"


class UsersRepository(Repository):

    def __init__(self, users_store):
        super(UsersRepository, self).__init__()
        self.users_store = users_store
        self.notifications = []
        notification_memory = Memory(on_add=self.users_store.insert_notification,
                                     on_edit=self.users_store.update_notification,
                                     on_remove=self.users_store.delete_notification,
                                     items=self.notifications
                                     )
        self.memories[USER_NOTIFICATIONS_MEMORY_KEY] = notification_memory

        self.user_settings = []
        user_settings_memory = Memory(on_add=self.users_store.insert_user_settings,
                                      on_edit=self.users_store.update_user_settings,
                                      on_remove=self.users_store.delete_user_settings,
                                      items=self.user_settings
                                      )
        self.memories[USER_SETTINGS_MEMORY_KEY] = user_settings_memory

        self.user_channels = []
        user_channels_memory = Memory(on_add=self.users_store.insert_user_channel,
                                      on_edit=None,
                                      on_remove=None,
                                      items=self.user_channels
                                      )
        self.memories[USER_CHANNELS_MEMORY_KEY] = user_channels_memory

    def get_user_channels(self, user_id):
        return self.users_store.get_user_channels(user_id)

    def get_user_settings(self, user_id):
        return self.users_store.fetch_user_settings(user_id)

    def get_notifications(self):
        return self.users_store.fetch_notifications()

    def add_notification(self, user_id, user_name, user_email, notification_type, check_every,is_active,
                         start_date, end_date, channel_type, threshold_value, source_id):
        n = user_notification(
            user_id=user_id, user_name=user_name, user_email=user_email,
            notification_type=notification_type,
            check_every=check_every,
            start_date=start_date,
            end_date=end_date,
            is_active=is_active,
            channel_type=channel_type, threshold_value=threshold_value, source_id=source_id,
            operation=OPERATIONS.ADDED.name)
        self.memories[USER_NOTIFICATIONS_MEMORY_KEY].items.append(n)
        return n

    def edit_notification(self, in_id, user_id, user_name, user_email, notification_type, check_every,
                          start_date,end_date, is_active, channel_type, threshold_value, source_id):
        n = user_notification(
            id=in_id,
            user_id=user_id, user_name=user_name,
            user_email=user_email, notification_type=notification_type,
            check_every=check_every, start_date=start_date, end_date=end_date, is_active=is_active,
            channel_type=channel_type, threshold_value=threshold_value, source_id=source_id,
            operation=OPERATIONS.MODIFIED.name)
        self.memories[USER_NOTIFICATIONS_MEMORY_KEY].items.append(n)
        return n

    def add_user_settings(self, user_id, preferred_currency, source_id):
        uc = user_settings(user_id=user_id, preferred_currency=preferred_currency, source_id=source_id,
                           operation=OPERATIONS.ADDED.name)
        self.user_settings.append(uc)
        return uc

    def edit_user_settings(self, user_id, preferred_currency, source_id):
        uc = user_settings(user_id=user_id,
                           preferred_currency=preferred_currency, source_id=source_id,

                           operation=OPERATIONS.MODIFIED.name)
        self.memories[USER_SETTINGS_MEMORY_KEY].items.append(uc)
        return uc

    def add_user_channel(self, user_id, channel_type, chat_id, source_id):
        uc = user_channel(
            user_id=user_id,
            channel_type=channel_type,
            chat_id=chat_id,
            operation=OPERATIONS.ADDED.name,
            source_id=source_id)
        self.memories[USER_CHANNELS_MEMORY_KEY].items.append(uc)
        return uc

    def remove_notification(self, in_id):
        self.mark_deleted(memory_key=USER_NOTIFICATIONS_MEMORY_KEY,
                          on_select=self.users_store.fetch_notification_by_id, id_value=in_id, id_name="id")

    def remove_user_setting(self, in_id):
        self.mark_deleted(memory_key=USER_CHANNELS_MEMORY_KEY,
                          on_select=self.users_store.fetch_user_settings_by_id, id_value=in_id, id_name="id")

    def remove_notification_by_source_id(self, source_id):
        self.mark_deleted(memory_key=USER_NOTIFICATIONS_MEMORY_KEY,
                          on_select=self.users_store.fetch_notification_by_id, id_value=source_id, id_name="source_id")



from abc import ABCMeta, abstractmethod


class TransactionStore(metaclass=ABCMeta):
    @abstractmethod
    def fetch_distinct_user_ids(self):
        pass

    @abstractmethod
    def fetch_distinct_user_ids(self):
        pass

    @abstractmethod
    def fetch_transaction(self, id):
        pass

    @abstractmethod
    def fetch_transactions(self, user_id):
        pass

    @abstractmethod
    def fetch_transactions(self, user_id):
        pass

    @abstractmethod
    def fetch_transactions_before_date(self, user_id, date):
        pass

    @abstractmethod
    def insert_transaction(self, user_id, volume, symbol, value, price, currency, date, source, source_id, operation):
        pass

    @abstractmethod
    def update_transaction(self, id, user_id, volume, symbol, value, price, currency, date, source, source_id,
                           operation):
        pass

    @abstractmethod
    def delete_transaction(self, id, throw_if_does_not_exist):
        pass

    @abstractmethod
    def delete_transaction_by_source_id(self, source_id, throw_if_does_not_exist):
        pass



"""
Represents an item list contained within Repository
keeps the set of methods executed for add/edit/delete
"""
TRANSACTIONS_MEMORY_KEY = "transactions"
USER_NOTIFICATIONS_MEMORY_KEY = "notifications"
USER_CHANNELS_MEMORY_KEY = "channels"
USER_SETTINGS_MEMORY_KEY = "settings"


class Memory:

    def __init__(self, on_add, on_edit, on_remove, items):
        self.on_add = on_add
        self.on_remove = on_remove
        self.on_edit = on_edit
        self.items = items
        self.handled_items = []



from cryptomodel.cryptostore import user_transaction
from cryptomodel.operations import OPERATIONS

from cryptodataaccess.Memory import Memory, TRANSACTIONS_MEMORY_KEY
from cryptodataaccess.Repository import Repository


class TransactionRepository(Repository):

    def __init__(self, transaction_store):
        self.transaction_store = transaction_store
        self.transactions = []
        super(TransactionRepository, self).__init__()
        memory = Memory(on_add=self.transaction_store.insert_transaction,
                        on_edit=self.transaction_store.update_transaction,
                        on_remove=self.transaction_store.delete_transaction,
                        items=self.transactions
                        )
        self.memories[TRANSACTIONS_MEMORY_KEY] = memory

    def get_distinct_user_ids(self):
        return self.transaction_store.fetch_distinct_user_ids()

    def get_transaction(self, id):
        return self.transaction_store.fetch_transaction(id)

    def get_transactions(self, user_id):
        return self.transaction_store.fetch_transactions(user_id)

    def get_transactions_before_date(self, user_id, date):
        return self.transaction_store.fetch_transactions_before_date(user_id, date)

    def add_transaction(self, user_id, volume, symbol, value, price, currency, date, source, source_id, transaction_type,
                        order_type):
        t = user_transaction(user_id=user_id, volume=volume, symbol=symbol, value=value, price=price,
                             currency=currency, date=date, source=source, source_id=source_id,
                             operation=OPERATIONS.ADDED.name, is_valid=True, invalid_reason="",
                             type=transaction_type, order_type = order_type
                             )
        self.memories[TRANSACTIONS_MEMORY_KEY].items.append(t)
        return t

    def edit_transaction(self, in_id, user_id, volume, symbol, value, price, currency, date, source, source_id, transaction_type,
                         order_type):
        t = user_transaction(id=in_id,
                             user_id=user_id, volume=volume, symbol=symbol, value=value, price=price,
                             currency=currency, date=date, source=source, source_id=source_id,
                             operation=OPERATIONS.MODIFIED.name,
                             type=transaction_type, order_type=order_type
                             )
        self.memories[TRANSACTIONS_MEMORY_KEY].items.append(t)
        return t

    def remove_transaction(self, in_id):
        self.mark_deleted(memory_key=TRANSACTIONS_MEMORY_KEY,
                          on_select=self.transaction_store.fetch_transaction, id_value=in_id, id_name="id")

    def remove_transaction_by_source_id(self, source_id):
        self.mark_deleted(memory_key=TRANSACTIONS_MEMORY_KEY,
                          on_select=self.transaction_store.fetch_transaction, id_value=source_id, id_name="source_id")



from datetime import datetime

import mock
from cryptomodel.coinmarket import prices
from cryptomodel.cryptomodel import user_transaction, exchange_rates

from cryptodataaccess.Rates.RatesMongoStore import RatesMongoStore
from cryptodataaccess.Rates.RatesRepository import RatesRepository
from cryptodataaccess.config import configure_app
import pytest
from cryptodataaccess.helpers import do_connect, convert_to_int_timestamp
from cryptodataaccess.tests.helpers import insert_prices_record, insert_exchange_record, insert_prices_record_with_method, \
    get_prices20200812039_record, get_prices20200801T2139_record

DATE_FORMAT = '%Y-%m-%d %H:%M:%S.%f'


@pytest.fixture(scope='module')
def mock_log():
    with mock.patch("cryptodataaccess.helpers.log_error"
                    ) as _mock:
        _mock.return_value = True
        yield _mock


def test_fetch_symbol_rates_for_date_pass_str_or_dt():
    config = configure_app()
    do_connect(config)
    user_transaction.objects.all().delete()
    exchange_rates.objects.all().delete()
    prices.objects.all().delete()

    insert_prices_record()
    insert_exchange_record()

    config = configure_app()
    store = RatesMongoStore(config, mock_log)
    rates_repo = RatesRepository(store)
    do_connect(config)

    rates_repo.fetch_symbol_rates_for_date(convert_to_int_timestamp(datetime.today()))

    dt = datetime(year=2040, month=1, day=1)
    rates_repo.fetch_symbol_rates_for_date(convert_to_int_timestamp(dt))

    assert (len(prices.objects) == 1)


def test_fetch_symbol_rates_for_dat_with_two_entries_within_two_hours():
    config = configure_app()
    users_store = RatesMongoStore(config, mock_log)
    rates_repo = RatesRepository(users_store)
    do_connect(config)
    dt_now = convert_to_int_timestamp(datetime.today())
    user_transaction.objects.all().delete()
    prices.objects.all().delete()

    insert_exchange_record()
    insert_prices_record_with_method(get_prices20200812039_record)
    insert_prices_record_with_method(get_prices20200801T2139_record)
    rts = rates_repo.fetch_symbol_rates_for_date(convert_to_int_timestamp(datetime.today()))
    dt = datetime(year=2020, month=8, day=1, hour=21, minute=0) #1596304800
    print(dt.timestamp()) #

    # assert (rts.rates['BTC'].last_updated == '2020-08-01T21:38:00.000Z')

    # dt = datetime(year=2020, month=8 , day=1, hour=21, minute=0 )
    # rts =      rates_repo.fetch_symbol_rates_for_date(convert_to_int_timestamp(dt))
    # assert(rts.rates['BTC'].last_updated =='2013-04-28T00:00:00.000Z' )



from mongoengine import Q
from cryptodataaccess.Transactions.TransactionStore import TransactionStore
from cryptodataaccess.helpers import if_none_raise_with_id
from cryptodataaccess import helpers
from cryptomodel.cryptostore import user_transaction


class TransactionMongoStore(TransactionStore):

    def __init__(self, config, log_error):
        self.configuration = config
        self.log_error = log_error

    def fetch_distinct_user_ids(self):
        return helpers.server_time_out_wrapper(self, self.do_fetch_transaction, id)

    def fetch_transaction(self, _id):
        return helpers.server_time_out_wrapper(self, self.do_fetch_transaction, _id)

    def fetch_transactions(self, user_id):
        return helpers.server_time_out_wrapper(self, self.do_fetch_transactions, user_id)

    def fetch_transactions_before_date(self, user_id, date):
        return helpers.server_time_out_wrapper(self, self.do_fetch_transactions_before_date, user_id, date)

    def fetch_transactions(self, user_id):
        return helpers.server_time_out_wrapper(self, self.do_fetch_transactions, user_id)

    def fetch_distinct_user_ids(self):
        return helpers.server_time_out_wrapper(self, self.do_fetch_distinct_user_ids)

    def insert_transaction(self, in_trans):
        return helpers.server_time_out_wrapper(self, self.do_insert_transaction, in_trans)

    def update_transaction(self, in_trans):
        return helpers.server_time_out_wrapper(self, self.do_update_transaction, in_trans)

    def delete_transaction(self, in_trans, throw_if_not_exist):
        helpers.server_time_out_wrapper(self, self.do_delete_transaction, in_trans, throw_if_not_exist)

    def do_delete_transaction(self, in_trans, throw_if_does_not_exist=True):
        helpers.do_connect(self.configuration)
        trans = user_transaction.objects(id=in_trans.id).first()
        if throw_if_does_not_exist:
            if_none_raise_with_id(in_trans.id, trans)
        if trans is not None:
            trans.delete()

    def delete_transaction_by_source_id(self, source_id, throw_if_does_not_exist=True):
        helpers.server_time_out_wrapper(self, self.do_delete_transaction, source_id, throw_if_does_not_exist)

    def do_delete_transaction_by_source_id(self, source_id, throw_if_does_not_exist=True):
        helpers.do_connect(self.configuration)
        trans = user_transaction.objects(source_id=source_id).first()
        if throw_if_does_not_exist:
            if_none_raise_with_id(id, trans)
        if trans is not None:
            trans.delete()

    def do_update_transaction(self, in_trans):
        helpers.do_connect(self.configuration)
        trans = user_transaction.objects(id=in_trans.id).first()
        if_none_raise_with_id(in_trans.id, trans)
        trans.user_id = in_trans.user_id
        trans.volume = in_trans.volume
        trans.symbol = in_trans.symbol
        trans.value = in_trans.value
        trans.price = in_trans.price
        trans.date = in_trans.date
        trans.currency = in_trans.currency
        trans.source = in_trans.source
        trans.source_id = in_trans.source_id
        trans.operation = in_trans.operation
        trans.type = in_trans.type
        trans.order_type = in_trans.order_type
        trans.save()
        return user_transaction.objects(id=trans.id).first()

    def do_insert_transaction(self, in_trans):
        helpers.do_connect(self.configuration)
        trans = user_transaction()
        trans.user_id = in_trans.user_id
        trans.volume = in_trans.volume
        trans.symbol = in_trans.symbol
        trans.value = in_trans.value
        trans.price = in_trans.price
        trans.date = in_trans.date
        trans.currency = in_trans.currency
        trans.source = in_trans.source
        trans.source_id = in_trans.source_id
        trans.operation = in_trans.operation
        trans.is_valid = in_trans.is_valid
        trans.type = in_trans.type
        trans.order_type = in_trans.order_type
        trans.invalid_reason = in_trans.invalid_reason
        trans.save()
        return user_transaction.objects(id=trans.id).first()

    def do_fetch_transactions(self, user_id):
        helpers.do_connect(self.configuration)
        return user_transaction.objects(Q(user_id=user_id))

    def do_fetch_transactions_before_date(self, user_id, date):
        helpers.do_connect(self.configuration)
        return user_transaction.objects(Q(user_id=user_id) &
                                        Q(date__lte=date)
                                        )

    def do_fetch_transaction(self, id):
        helpers.do_connect(self.configuration)
        trans = user_transaction.objects(Q(id=id))
        if len(trans) == 1:
            return user_transaction.objects(Q(id=id))[0]
        return None

    def do_fetch_distinct_user_ids(self):
        helpers.do_connect(self.configuration)
        return user_transaction.objects().only('user_id').distinct('user_id')



from datetime import datetime

import mock
from bson import ObjectId
from cryptomodel.operations import OPERATIONS
from cryptomodel.cryptostore import user_channel, user_notification, user_settings
from cryptomodel.coinmarket import prices
from cryptomodel.fixer import exchange_rates

from cryptodataaccess.Memory import USER_NOTIFICATIONS_MEMORY_KEY, USER_SETTINGS_MEMORY_KEY, USER_CHANNELS_MEMORY_KEY
from cryptodataaccess.Rates.RatesMongoStore import RatesMongoStore
from cryptodataaccess.Users.UsersMongoStore import UsersMongoStore
from cryptodataaccess.config import configure_app
from cryptodataaccess.Users.UsersRepository import UsersRepository
from cryptodataaccess.Rates.RatesRepository import RatesRepository
import pytest

from cryptodataaccess.helpers import convert_to_int_timestamp
from cryptodataaccess.tests.helpers import insert_prices_record, insert_exchange_record
from cryptodataaccess import helpers


@pytest.fixture(scope='module')
def mock_log():
    with mock.patch("cryptodataaccess.helpers.log_error"
                    ) as _mock:
        _mock.return_value = True
        yield _mock


def test_fetch_symbol_rates():
    config = configure_app()
    rates_store = RatesMongoStore(config, mock_log)
    repo = RatesRepository(rates_store)

    helpers.do_connect(config)
    prices.objects.all().delete()
    insert_prices_record()
    objs = repo.fetch_symbol_rates()
    assert (len(objs.rates) == 100)
    assert (objs.rates['BTC'].price == 8101.799293468747)


def test_fetch_exchange_rates():
    config = configure_app()
    rates_store = RatesMongoStore(config, mock_log)
    repo = RatesRepository(rates_store)

    exchange_rates.objects.all().delete()
    insert_exchange_record()
    objs = repo.fetch_latest_exchange_rates_to_date('1900-01-01')
    assert (len(objs) == 0)
    objs = repo.fetch_latest_exchange_rates_to_date('2020-07-04')
    assert (len(objs) == 1)
    objs = repo.fetch_latest_exchange_rates_to_date('2020-07-03')
    assert (len(objs) == 1)
    assert (objs[0].rates.AED == 4.127332)
    objs = repo.fetch_latest_exchange_rates_to_date('2020-07-02')
    assert (len(objs) == 0)


def test_fetch_prices_and_symbols():
    config = configure_app()
    rates_store = RatesMongoStore(config, mock_log)
    repo = RatesRepository(rates_store)

    prices.objects.all().delete()
    insert_prices_record()

    dt = convert_to_int_timestamp(datetime(year=2020, month = 7 , day = 3 ))
    objs = repo.fetch_latest_prices_to_date(dt)
    assert (len(objs) == 1)
    objs = repo.fetch_latest_prices_to_date(dt)

    dt = convert_to_int_timestamp(datetime(year=2020, month=7, day=4))


    assert (len(objs) == 1)
    symbols = repo.fetch_symbols()
    assert (len(symbols) == 100)


def test_insert_user_channel():
    config = configure_app()
    rates_store = UsersMongoStore(config, mock_log)
    repo = UsersRepository(rates_store)
    helpers.do_connect(config)

    user_channel.objects.all().delete()
    repo.add_user_channel(user_id=1, chat_id='1', channel_type='telegram',
                          source_id=ObjectId('666f6f2d6261722d71757578'))
    repo.commit()
    uc = repo.memories[USER_CHANNELS_MEMORY_KEY].items[0]
    assert (uc.channel_type == 'telegram')
    assert (uc.operation == OPERATIONS.ADDED.name)


def test_insert_user_setting():
    config = configure_app()
    users_store = UsersMongoStore(config, mock_log)
    repo = UsersRepository(users_store)
    helpers.do_connect(config)
    user_settings.objects.all().delete()
    repo.add_user_settings(user_id=1, preferred_currency='da', source_id=ObjectId('666f6f2d6261722d71757578'))
    repo.commit()
    uc = repo.memories[USER_SETTINGS_MEMORY_KEY].items[0]
    assert (uc.preferred_currency == 'da')
    assert (uc.operation == OPERATIONS.ADDED.name)


def test_update_notification_when_does_not_exist_throws_ValueError():
    config = configure_app()
    store = UsersMongoStore(config, mock_log)
    repo = UsersRepository(store)

    helpers.do_connect(config)

    user_notification.objects.all().delete()
    with pytest.raises(ValueError):
        repo.edit_notification(
            in_id=ObjectId('666f6f2d6261722d71757578'),
            channel_type="TELEGRAM",
            notification_type="BALANCE",
            user_id=1,
            user_name="test",
            user_email="ds",
            is_active=False,
            source_id=ObjectId('666f6f2d6261722d71757578'),
            check_every="00:00",
            start_date=datetime.now(),
            end_date= datetime.now(),
            threshold_value=1

        )
        repo.commit()


def test_update_notification():
    config = configure_app()
    store = UsersMongoStore(config, mock_log)
    repo = UsersRepository(store)

    helpers.do_connect(config)

    user_notification.objects.all().delete()
    repo.add_notification(user_id=1, user_name='username', user_email='email',
                          notification_type='BALANCE',
                          is_active=True, channel_type='TELEGRAM',
                          threshold_value=1,
                          check_every="00:00",
                          start_date=datetime.now(),
                          end_date=datetime.now(),
                          source_id=ObjectId('666f6f2d6261722d71757578'))
    repo.commit()
    un = repo.memories[USER_NOTIFICATIONS_MEMORY_KEY].items[0]

    repo.edit_notification(in_id=un.id,
                           user_id=1, user_name='username2', user_email='email',
                           notification_type='BALANCE',     check_every="00:00",
                           start_date=datetime.now(),
                           end_date=datetime.now(),
                           is_active=True, channel_type='TELEGRAM',
                           threshold_value=1,
                           source_id=ObjectId('666f6f2d6261722d71757578'))
    repo.commit()
    un = repo.memories[USER_NOTIFICATIONS_MEMORY_KEY].items[0]

    assert (un.user_name == "username2")


def test_delete_notification_when_exists():
    config = configure_app()
    store = UsersMongoStore(config, mock_log)
    repo = UsersRepository(store)

    helpers.do_connect(config)

    user_notification.objects.all().delete()

    repo.add_notification(user_id=1, user_name='username', user_email='email',
                          threshold_value=1,
                          notification_type = "BALANCE",
                          check_every="00:00",
                          start_date=datetime.now(),
                          end_date=datetime.now(),

                          is_active=True, channel_type='TELEGRAM', source_id=ObjectId('666f6f2d6261722d71757578'))
    repo.commit()
    ut = repo.memories[USER_NOTIFICATIONS_MEMORY_KEY].items[0]

    assert (len(user_notification.objects) == 1)
    ut = repo.remove_notification(ut.id)
    repo.commit()
    assert (len(user_notification.objects) == 0)


def test_delete_user_notification_when_exists_by_source_id():
    config = configure_app()
    store = UsersMongoStore(config, mock_log)
    repo = UsersRepository(store)

    helpers.do_connect(config)

    user_notification.objects.all().delete()

    repo.add_notification(user_id=1, user_name='username', user_email='email',
                          expression_to_evaluate='some expr', check_every_seconds=1, check_times=1,
                          is_active=True, channel_type='telegram',
                          fields_to_send="dsd",
                          source_id=ObjectId('666f6f2d6261722d71757578'))
    repo.commit()
    ut = repo.memories[USER_NOTIFICATIONS_MEMORY_KEY].items[0]
    assert (len(user_notification.objects) == 1)
    store.do_delete_user_notification_by_source_id(source_id=ObjectId('666f6f2d6261722d71757578'))
    assert (len(user_notification.objects) == 0)


def test_delete_user_notification_when_exists_by_source_id():
    config = configure_app()
    store = UsersMongoStore(config, mock_log)
    repo = UsersRepository(store)

    helpers.do_connect(config)

    user_notification.objects.all().delete()

    repo.add_notification(user_id=1, user_name='username', user_email='email',
                          notification_type='BALANCE',
                          is_active=True, channel_type='TELEGRAM',
                          threshold_value=1,
                          check_every="00:00",
                          start_date=datetime.now(),
                          end_date=datetime.now(),
                          source_id=ObjectId('666f6f2d6261722d71757578'))
    repo.commit()
    ut = repo.memories[USER_NOTIFICATIONS_MEMORY_KEY].items[0]
    assert (len(user_notification.objects) == 1)
    store.do_delete_user_notification_by_source_id(source_id=ObjectId('666f6f2d6261722d71757578'))
    assert (len(user_notification.objects) == 0)


def test_Users_Repository_create():
    config = configure_app()
    store = UsersMongoStore(config, mock_log)
    repo = UsersRepository(store)

    assert (repo.memories[USER_NOTIFICATIONS_MEMORY_KEY] is not None)
    assert (repo.memories[USER_NOTIFICATIONS_MEMORY_KEY].items is not None)


def test_do_delete_notification_when_does_not_exist_should_not_throw():
    config = configure_app()
    store = UsersMongoStore(config, mock_log)

    noti = user_notification()
    noti.id = ObjectId('666f6f2d6261722d71757578')
    noti.source_id =ObjectId('666f6f2d6261722d71757578')
    store.delete_notification(notification=noti,throw_if_does_not_exist=False)
    assert(1==1)


def test_remove_notification_by_source_id_should_not_add_any_object_to_memory_when_does_not_exist_in_db_and_mem():
    config = configure_app()
    store = UsersMongoStore(config, mock_log)
    repo = UsersRepository(store)
    helpers.do_connect(config)
    user_notification.objects.all().delete()

    store.do_delete_user_notification_by_source_id(source_id= ObjectId('666f6f2d6261722d71757578'), throw_if_does_not_exist=False)
    assert (len(repo.memories[USER_NOTIFICATIONS_MEMORY_KEY].items)==0)

def test_delete_notification_by_source_id_should_not_add_any_object_to_memory_when_does_not_exist_in_db_and_mem():
    config = configure_app()
    store = UsersMongoStore(config, mock_log)
    repo = UsersRepository(store)
    helpers.do_connect(config)
    user_notification.objects.all().delete()

    store.do_delete_user_notification_by_source_id(source_id= ObjectId('666f6f2d6261722d71757578'), throw_if_does_not_exist=False)
    assert (len(repo.memories[USER_NOTIFICATIONS_MEMORY_KEY].items)==0)


def test_remove_notification_by_source_id_should_not_add_any_object_to_memory_when_does_not_exist_in_db_and_mem():
    config = configure_app()
    store = UsersMongoStore(config, mock_log)
    repo = UsersRepository(store)
    helpers.do_connect(config)
    user_notification.objects.all().delete()

    repo.remove_notification_by_source_id(source_id=ObjectId('666f6f2d6261722d71757578'))
    assert (len(repo.memories[USER_NOTIFICATIONS_MEMORY_KEY].items) == 0)




from datetime import datetime

import mock
from bson import ObjectId
from cryptomodel.cryptostore import user_transaction
from cryptomodel.operations import OPERATIONS

from cryptodataaccess.Memory import TRANSACTIONS_MEMORY_KEY
from cryptodataaccess.Transactions.TransactionMongoStore import TransactionMongoStore
from cryptodataaccess.config import configure_app
from cryptodataaccess.Transactions.TransactionRepository import TransactionRepository
import pytest
from cryptodataaccess.helpers import do_connect
from mongoengine import Q


@pytest.fixture(scope='module')
def mock_log():
    with mock.patch("cryptodataaccess.helpers.log_error"
                    ) as _mock:
        _mock.return_value = True
        yield _mock


def test_insert_transaction():
    config = configure_app()
    store = TransactionMongoStore(config, mock_log)
    repo = TransactionRepository(store)
    do_connect(config)
    user_transaction.objects.all().delete()
    ut = repo.add_transaction(1, 1, 'OXT', 1, 1, "USD", "2020-01-01", "kraken",
                              source_id=ObjectId('666f6f2d6261722d71757578'), transaction_type="TRADE", order_type="BUY")
    repo.commit()

    ut = repo.memories[TRANSACTIONS_MEMORY_KEY].items[0]
    assert (ut.user_id == 1)
    assert (ut.symbol == "OXT")
    assert (len(user_transaction.objects) == 1)
    assert (ut.operation == OPERATIONS.ADDED.name)


def test_update_transaction():
    config = configure_app()
    store = TransactionMongoStore(config, mock_log)
    repo = TransactionRepository(store)
    do_connect(config)
    user_transaction.objects.all().delete()
    repo.add_transaction(1, 1, 'OXT', 1, 1, "USD", "2020-01-01", "kraken",
                         source_id=ObjectId('666f6f2d6261722d71757578'), transaction_type="TRADE", order_type="BUY")

    assert (len(repo.memories) == 1)
    assert (len(repo.memories[TRANSACTIONS_MEMORY_KEY].items) == 1)

    repo.commit()
    ut = repo.memories[TRANSACTIONS_MEMORY_KEY].items[0]

    repo.edit_transaction(ut.id, 1, 1, 'OXT2', 1, 1, "EUR", "2020-01-01", "kraken",
                          source_id=ObjectId('666f6f2d6261722d71757578'), transaction_type="TRADE", order_type="BUY")
    repo.commit()
    ut = repo.memories[TRANSACTIONS_MEMORY_KEY].items[0]

    assert (ut.user_id == 1)
    assert (ut.symbol == "OXT2")
    assert (ut.currency == "EUR")
    assert (ut.operation == OPERATIONS.MODIFIED.name)


def test_update_transaction_when_does_not_exist_throws_ValueError():
    config = configure_app()
    store = TransactionMongoStore(config, mock_log)
    repo = TransactionRepository(store)
    do_connect(config)
    user_transaction.objects.all().delete()
    with pytest.raises(ValueError):
        repo.edit_transaction(ObjectId('666f6f2d6261722d71757578'), 1, 1, 'OXT', "EUR", 1, 1, "2020-01-01",
                              "kraken", source_id=ObjectId('666f6f2d6261722d71757578'), transaction_type="TRADE", order_type="BUY")
        repo.commit()

def test_delete_transaction_when_does_not_exist_throws_ValueError():
    config = configure_app()
    store = TransactionMongoStore(config, mock_log)
    repo = TransactionRepository(store)
    do_connect(config)
    user_transaction.objects.all().delete()
    trans = user_transaction()
    trans.id = ObjectId('666f6f2d6261722d71757578')
    with pytest.raises(ValueError):
        store.delete_transaction(trans, True)


def test_delete_transaction_when_does_not_exist_and_throw_is_false_does_not_throw():
    config = configure_app()
    store = TransactionMongoStore(config, mock_log)
    repo = TransactionRepository(store)
    do_connect(config)
    user_transaction.objects.all().delete()
    trans = user_transaction()
    trans.id = ObjectId('666f6f2d6261722d71757578')
    store.delete_transaction(trans, False)


def test_delete_transaction_when_exists():
    config = configure_app()
    store = TransactionMongoStore(config, mock_log)
    repo = TransactionRepository(store)
    do_connect(config)
    repo.add_transaction(1, 1, 'OXT', 1, 1, "EUR", "2020-01-01", "kraken",
                         source_id=ObjectId('666f6f2d6261722d71757578'), transaction_type="TRADE", order_type="BUY")
    repo.commit()
    assert (len(user_transaction.objects) == 1)

    ut = repo.memories[TRANSACTIONS_MEMORY_KEY].items[0]

    repo.remove_transaction(ut.id)
    repo.commit()

    assert (len(user_transaction.objects) == 0)


def test_delete_transaction_when_exists_by_source_id():
    config = configure_app()
    store = TransactionMongoStore(config, mock_log)
    repo = TransactionRepository(store)
    do_connect(config)
    user_transaction.objects.all().delete()

    ut = repo.add_transaction(1, 1, 'OXT', 1, 1, "EUR", "2020-01-01", "kraken",
                              source_id=ObjectId('666f6f2d6261722d71757578'), transaction_type="TRADE", order_type="BUY")
    repo.commit()
    assert (len(user_transaction.objects) == 1)
    ut = repo.memories[TRANSACTIONS_MEMORY_KEY].items[0]
    repo.remove_transaction_by_source_id(source_id=ut.source_id)
    repo.commit()
    assert (len(user_transaction.objects) == 0)


def test_fetch_transaction():
    config = configure_app()
    store = TransactionMongoStore(config, mock_log)
    repo = TransactionRepository(store)
    do_connect(config)
    user_transaction.objects.all().delete()
    ut = repo.add_transaction(1, 1, 'OXT', 1, 1, "USD", "2020-01-01", "kraken",
                              source_id=ObjectId('666f6f2d6261722d71757578'), transaction_type="TRADE", order_type="BUY")
    repo.commit()

    ut = repo.memories[TRANSACTIONS_MEMORY_KEY].items[0]
    assert (ut.user_id == 1)
    assert (ut.symbol == "OXT")
    assert (ut.currency == "USD")
    assert (ut.operation == OPERATIONS.ADDED.name)
    assert (ut.source_id == ObjectId('666f6f2d6261722d71757578'))
    repo.add_transaction(1, 1, 'OXT', 1, 1, "USD", "2020-01-01", "kraken",
                         source_id=None, transaction_type="TRADE", order_type="BUY")
    repo.commit()
    ut = repo.memories[TRANSACTIONS_MEMORY_KEY].items[0]
    ut = repo.get_transaction(ut.id)
    assert (ut.source_id is None)


def test_fetch_distinct_user_ids():
    config = configure_app()
    store = TransactionMongoStore(config, mock_log)
    repo = TransactionRepository(store)
    do_connect(config)
    user_transaction.objects.all().delete()
    repo.add_transaction(1, 1, 'OXT', 1, 1, "USD", "2020-01-01", "kraken",
                         source_id=ObjectId('666f6f2d6261722d71757578'), transaction_type="TRADE", order_type="BUY")

    repo.commit()
    items = repo.get_distinct_user_ids()
    assert (len(items) == 1)
    user_transaction.objects.all().delete()
    repo.add_transaction(1, 1, 'OXT', 1, 1, "USD", "2020-01-01", "kraken",
                         source_id=ObjectId('666f6f2d6261722d71757578'), transaction_type="TRADE", order_type="BUY")
    repo.add_transaction(12, 1, 'OXT', 1, 1, "USD", "2020-01-01", "kraken",
                         source_id=ObjectId('666f6f2d6261722d71757578'), transaction_type="TRADE", order_type="BUY")

    repo.commit()
    items = repo.get_distinct_user_ids()
    assert (len(items) == 2)
    user_transaction.objects.all().delete()
    repo.add_transaction(1, 1, 'OXT', 1, 1, "USD", "2020-01-01", "kraken",
                         source_id=ObjectId('666f6f2d6261722d71757578'), transaction_type="TRADE", order_type="BUY")
    repo.add_transaction(12, 1, 'OXT', 1, 1, "USD", "2020-01-01", "kraken",
                         source_id=ObjectId('666f6f2d6261722d71757578'), transaction_type="TRADE", order_type="BUY")
    repo.add_transaction(3, 1, 'OXT', 1, 1, "USD", "2020-01-01", "kraken",
                         source_id=ObjectId('666f6f2d6261722d71757578'), transaction_type="TRADE", order_type="BUY")
    repo.add_transaction(122, 1, 'OXT', 1, 1, "USD", "2020-01-01", "kraken",
                         source_id=ObjectId('666f6f2d6261722d71757578'), transaction_type="TRADE", order_type="BUY")
    repo.commit()
    items = repo.get_distinct_user_ids()
    assert (len(items) == 4)


def test_get_transaction_by_date():
    config = configure_app()
    store = TransactionMongoStore(config, mock_log)
    repo = TransactionRepository(store)
    do_connect(config)
    user_transaction.objects.all().delete()

    repo.add_transaction(1, 1, 'OXT', 1, 1, "EUR", "2020-01-01", "kraken",
                              source_id=ObjectId('666f6f2d6261722d71757578'), transaction_type="TRADE", order_type="BUY")
    repo.commit()
    assert (len(user_transaction.objects) == 1)

    ts = repo.get_transactions_before_date(1,"2019-01-01")
    assert (len(ts) == 0)

    ts = repo.get_transactions_before_date(1,"2020-01-01")
    assert (len(ts) == 1)

    ts = repo.get_transactions_before_date(1,"2019-12-31")
    assert (len(ts) == 0)

    dt_now = datetime.today()

    ts = repo.get_transactions_before_date(1,dt_now)
    assert (len(ts) == 1)




def test_remove_and_add():
    config = configure_app()
    store = TransactionMongoStore(config, mock_log)
    repo = TransactionRepository(store)
    do_connect(config)
    user_transaction.objects.all().delete()



    repo.remove_transaction_by_source_id( source_id=ObjectId('666f6f2d6261722d71757578'))

    repo.add_transaction(1, 1, 'OXT', 1, 1, "EUR", "2020-01-01", "kraken",
                         source_id=ObjectId('666f6f2d6261722d71757578'), transaction_type="TRADE", order_type="BUY")

    repo.commit()

    assert (len(user_transaction.objects) == 1)



import mongoengine
from mongoengine import connect, Q
from pymongo.errors import ServerSelectionTimeoutError
import logging


def do_connect(configuration):
    url = 'mongodb://' + configuration.USERNAME + ':' + configuration.PASSWORD + '@' \
          + configuration.SERVERNAME + ':' + str(configuration.PORT) + '/?authSource=admin'
    conn = connect(db=configuration.DATABASE, username=configuration.USERNAME, host=url)


def get_url(configuration):
    return 'mongodb://' + configuration.USERNAME + ':' + configuration.PASSWORD + '@' \
           + configuration.SERVERNAME + ':' + str(configuration.PORT) + '/?authSource=admin'


def server_time_out_wrapper(repository, method_to_call, *args):
    try:
        return method_to_call(*args)
    except ServerSelectionTimeoutError as inst:
        repository.log_error(exception=inst, web_method_name=str(method_to_call), cfg=repository.configuration)
        raise
    except mongoengine.connection.ConnectionFailure as cf:
        repository.log_error(exception=cf, web_method_name=str(method_to_call), cfg=repository.configuration)
        raise mongoengine.connection.ConnectionFailure(cf)


def convert_to_int_timestamp(dt):
    return int(dt.timestamp()) * 1000


def if_none_raise_with_id(_id, trans):
    if trans is None:
        raise ValueError(str(_id) + " does not exist ")


def if_none_raise(trans):
    if trans is None:
        raise ValueError(" does not exist ")


def if_empty_string_raise(value):
    if value is None:
        raise ValueError(" does not exist ")
    if value == '':
        raise ValueError(" does not exist ")


def log_error(exception, pk_id, web_method_name, cfg):
    logging.basicConfig(filename=cfg.LOGS_PATH, level=logging.ERROR)
    logging.error(str(pk_id) + ' ' + str(web_method_name) + ' ' + str(exception))



from datetime import datetime
from cryptomodel.coinmarket import prices
from cryptomodel.fixer import exchange_rates
from cryptomodel.readonly import SymbolRates
from mongoengine import Q
from cryptodataaccess.helpers import server_time_out_wrapper, do_connect

DATE_FORMAT = "%Y-%m-%d"


class CalculatorRepository:

    def __init__(self, config, log_error):
        self.configuration = config
        self.log_error = log_error

    def evaluate_variable(self):
        pass

    def evaluate_expression(self):
        pass


class StatementExecutor:

    def prepare(self):
        pass

    def execute(self):
        pass


class MongoEngineStatementExecutor(StatementExecutor):
    pass



from abc import ABCMeta, abstractmethod


class RatesStore(metaclass=ABCMeta):
    @abstractmethod
    def fetch_symbols(self):
        pass

    @abstractmethod
    def fetch_symbol_rates_for_date(self, dt):
        pass

    @abstractmethod
    def fetch_symbol_rates(self):
        pass

    @abstractmethod
    def fetch_latest_prices_to_date(self, before_date):
        pass

    @abstractmethod
    def fetch_latest_exchange_rates_to_date(self, before_date):
        pass



from cryptomodel.operations import OPERATIONS

MEMORY_HANDLED_ATTRIBUTE_NAME = "memory_handled"

class Repository:

    def __init__(self):
        self.memories = {}

    def commit(self):
        for key, mem in self.memories.items():

            self.remove_handled_items(mem.items)
            for item in mem.items:
                if not getattr(item, MEMORY_HANDLED_ATTRIBUTE_NAME):
                    if item.operation == OPERATIONS.ADDED.name:
                        trans = mem.on_add(item)
                        if trans is not None:
                            item.id = trans.id
                            setattr(item, MEMORY_HANDLED_ATTRIBUTE_NAME, True)
                    elif item.operation == OPERATIONS.MODIFIED.name:
                        trans = mem.on_edit(item)
                        if trans is not None:
                            item.id = trans.id
                            setattr(item, MEMORY_HANDLED_ATTRIBUTE_NAME, True)
                    elif item.operation == OPERATIONS.REMOVED.name:
                        mem.on_remove(item, False)
                        setattr(item,MEMORY_HANDLED_ATTRIBUTE_NAME, True)


    def remove_handled_items(self, source_list):
       for item in list(source_list):
           if not hasattr(item, MEMORY_HANDLED_ATTRIBUTE_NAME):
               setattr(item, MEMORY_HANDLED_ATTRIBUTE_NAME, False)
           if getattr(item, MEMORY_HANDLED_ATTRIBUTE_NAME):
                source_list.remove(item)

    """
    Assumes item has two attributes named as follows  
    operation  values : ADDED, REMOVED , MODIFIED 
    """

    def mark_deleted(self, memory_key, on_select, id_value, id_name):
        item = next((x for x in self.memories[memory_key].items if getattr(x, id_name) == id_value), None)
        if item is None:
            try:
                if len(on_select(id_value)) == 0:
                    return
                trans = on_select(id_value)[:1]
            except:
                trans = None  # log ?
            if trans is None:
                return
            else:
                trans.operation = OPERATIONS.REMOVED.name
                self.memories[memory_key].items.append(trans)
        else:
            item.operation = OPERATIONS.REMOVED.name
            setattr(item, MEMORY_HANDLED_ATTRIBUTE_NAME,
                    False)  # add, commit, remove sequence should set back to handled = false to allow deletion




from abc import ABCMeta, abstractmethod


class UsersStore(metaclass=ABCMeta):

    @abstractmethod
    def fetch_user_channels(self, user_id):
        pass

    @abstractmethod
    def fetch_notifications(self):
        pass

    @abstractmethod
    def fetch_notification_by_id(self, id):
        pass

    @abstractmethod
    def insert_notification(self, user_id, user_name, user_email, expression_to_evaluate, check_every_seconds,
                            check_times, is_active, channel_type, fields_to_send, source_id, operation):
        pass

    @abstractmethod
    def update_notification(self, id, user_id, user_name, user_email, expression_to_evaluate, check_every_seconds,
                            check_times, is_active, channel_type, fields_to_send, source_id, operation):
        pass

    @abstractmethod
    def fetch_user_settings_by_id(self, id):
        pass

    @abstractmethod
    def update_user_settings(self, user_id, preferred_currency):
        pass

    @abstractmethod
    def insert_user_settings(self, user_id, preferred_currency):
        pass

    @abstractmethod
    def fetch_user_channel_by_id(self, id):
        pass

    @abstractmethod
    def insert_user_channel(self, user_id, channel_type, chat_id):
        pass

    @abstractmethod
    def delete_notification(self, id):
        pass

    @abstractmethod
    def delete_user_settings(self, id):
        pass



from cryptodataaccess.Repository import Repository

DATE_FORMAT = "%Y-%m-%d"


class RatesRepository(Repository):

    def __init__(self, rates_store):
        self.rates_store = rates_store
        super(RatesRepository, self).__init__()

    def fetch_symbols(self):
        return self.rates_store.fetch_symbols()

    def fetch_symbol_rates(self):
        return self.rates_store.fetch_symbol_rates()

    def fetch_symbol_rates_for_date(self, dt):
        return self.rates_store.fetch_symbol_rates_for_date(dt)

    def fetch_latest_prices_to_date(self, before_date):
        return self.rates_store.fetch_latest_prices_to_date(before_date)

    def fetch_latest_exchange_rates_to_date(self, before_date):
        return self.rates_store.fetch_latest_exchange_rates_to_date(before_date)



from datetime import datetime
from cryptomodel.coinmarket import prices
from cryptomodel.fixer import exchange_rates, Q
from cryptomodel.readonly import SymbolRates
from cryptodataaccess.Rates.RatesStore import RatesStore
from cryptodataaccess.helpers import server_time_out_wrapper, do_connect, convert_to_int_timestamp

DATE_FORMAT = '%Y-%m-%d %H:%M:%S.%f'


class RatesMongoStore(RatesStore):

    def __init__(self, config, log_error):
        self.configuration = config
        self.log_error = log_error

    def fetch_symbols(self):
        symbols = {}
        latest_prices = self.fetch_latest_prices_to_date( convert_to_int_timestamp(datetime.today()))
        for coin in latest_prices[0].coins:
            symbols.update({coin.symbol: coin.name})
        return symbols

    def fetch_symbol_rates(self):
        dt_now = convert_to_int_timestamp(datetime.today())
        return self.fetch_symbol_rates_for_date(dt_now)

    def fetch_symbol_rates_for_date(self, dt):

        srs = SymbolRates(dt)
        latest_prices = self.fetch_latest_prices_to_date(dt)
        for coin in latest_prices[0].coins:
            srs.add_rate(coin.symbol, coin.quote.eur)
        return srs

    def fetch_latest_prices_to_date(self, before_date):
        return server_time_out_wrapper(self, self.do_fetch_latest_prices_to_date, before_date)

    def fetch_latest_exchange_rates_to_date(self, before_date):
        return server_time_out_wrapper(self, self.do_fetch_latest_exchange_rates_to_date, before_date)

    def do_fetch_latest_prices_to_date(self, before_date):
        do_connect(self.configuration)
        return prices.objects(Q(status__timestamp__lte=before_date)).order_by(
            '-status__timestamp')[:10]

    def do_fetch_latest_exchange_rates_to_date(self, before_date):
        do_connect(self.configuration)
        return exchange_rates.objects(Q(date__lte=before_date)).order_by(
            'date-')[:1]



import os
from keyring import get_password
from werkzeug.utils import import_string

DB = "calculator_service"
PORT = 27017
MONGO_IP = "134.122.79.43"
KAFKA_BROKERS = "192.168.1.57:9092"
TRANSACTIONS_TOPIC_NAME = "transactions_2"


class BaseConfig(object):
    DEBUG = False
    TESTING = False
    SERVERNAME = "localhost"
    PORT = PORT
    DATABASE = DB
    USERNAME = ""
    PASSWORD = ""
    LOGS_PATH = '../cryptodataaccess/logs/cryptodataaccess.log'
    KAFKA_BROKERS = KAFKA_BROKERS
    TRANSACTIONS_TOPIC_NAME = TRANSACTIONS_TOPIC_NAME


class DevelopmentConfig(BaseConfig):
    DEBUG = True
    TESTING = True
    SERVERNAME = "localhost"
    PORT = PORT
    DATABASE = DB
    USERNAME = "test"
    PASSWORD = "test"
    LOGS_PATH = '../cryptodataaccess/logs/cryptodataaccess.log'
    KAFKA_BROKERS = KAFKA_BROKERS
    TRANSACTIONS_TOPIC_NAME = TRANSACTIONS_TOPIC_NAME


class ProductionConfig(BaseConfig):
    DEBUG = False
    TESTING = False
    SERVERNAME = MONGO_IP
    PORT = PORT
    DATABASE = DB
    USERNAME = ""
    PASSWORD = ""
    LOGS_PATH = '../cryptodataaccess/logs/cryptodataaccess.log'
    KAFKA_BROKERS = KAFKA_BROKERS
    TRANSACTIONS_TOPIC_NAME = TRANSACTIONS_TOPIC_NAME


config = {
    "development": "cryptodataaccess.config.DevelopmentConfig",
    "production": "cryptodataaccess.config.ProductionConfig",
    "default": "cryptodataaccess.config.DevelopmentConfig",
}


def configure_app():
    config_name = os.getenv('FLASK_ENV', "cryptodataaccess.config.DevelopmentConfig")
    cfg = import_string(config_name)()
    cfg.USERNAME = get_password('cryptodataaccess', 'USERNAME')
    cfg.PASSWORD = get_password('cryptodataaccess', cfg.USERNAME)
    return cfg
